using System;
using System.Text;
using System.IO;
using System.Net;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Windows.Forms;

namespace CTCShell
{
    public partial class MainForm : Form
    {
        // All Global Constants
        public const string DEF_PORT = "TCP";
        public const string DEF_BAUD = "19200";
        public const string DEF_IPADD = "10.0.0.204";
        //        public const string DEF_IPADD = "172.16.2.85";
        public const string DEF_TIMEOUT = "1000";

        public const int FAILURE = 0;
        public const int SUCCESS = 1;
        
        public const int REG_OS = 13003;
        public const int REG_MILLI = 13002;
        public const int REG_RTCLOCK = 13013;
        public const int REG_SECONDS = 13014;
        public const int REG_MINUTES = 13015;
        public const int REG_HOURS = 13016;
        public const int REG_DAY = 13017;
        public const int REG_MONTH = 13018;
        public const int REG_YEAR = 13019;
        public const int REG_DOW = 13020;
        public const int REG_DOUTS = 1001;
        public const int REG_DOUTS8 = 10201;
        public const int REG_DOUTS16 = 10101;
        public const int REG_DOUTS32 = 10001;
        public const int REG_DINS = 2001;
        public const int REG_DINS8 = 11201;
        public const int REG_DINS16 = 11101;
        public const int REG_DINS32 = 11001;
        
        public int CTconnection;
        public const int rspTimeout = 1000;

        // All Code
        public MainForm()
        {
            InitializeComponent();

            // Set the communication defaults (hardcoded to keep things simple).
            statPort.Text = DEF_PORT;
            statBaud.Text = DEF_BAUD;
            statIPAdd.Text = DEF_IPADD;
            statTimeout.Text = DEF_TIMEOUT;
            statStatus.Text = "Disconnected";
        }

        private void toolConnect_Click(object sender, EventArgs e)
        {
            // Call the function that connects to the controller.
            Connect();
        }
        private unsafe void Connect()
        {
            Int32 timeout;
            // Are we currently disconnected?
            if (statStatus.Text == "Disconnected")
            {
                // Make the proper type connection
                if (statPort.Text == "UDP")
                {
                    // It's a UDP connection...
                    CTconnection = Ctccomm32v2.CtOpenConnection(Ctccomm32v2.CTCUDP, 0, Ctccomm32v2.inet_addr(statIPAdd.Text));
                }
                else if (statPort.Text == "TCP")
                {
                    // It's a TCP connection
                    CTconnection = Ctccomm32v2.CtOpenConnection(Ctccomm32v2.CTCTCP, 0, Ctccomm32v2.inet_addr(statIPAdd.Text));
                }
                else
                {
                    // It's a serial connection
                    CTconnection = Ctccomm32v2.CtOpenConnection(Ctccomm32v2.SERIAL, convertPort(statPort.Text), 1);
                    Ctccomm32v2.CtInitConnection(CTconnection, convertBAUD(statBaud.Text), Ctccomm32v2.NO_PARITY, Ctccomm32v2.EIGHT_DATA_BITS, Ctccomm32v2.ONE_STOP_BITS);
                }
                // Can we convert it to an integer?
                if (Int32.TryParse(statTimeout.Text, out timeout) == true)
                {
                    // Yes, then do it!
                    Ctccomm32v2.CtSetConnectionTimeout(CTconnection, timeout);
                }
                // Test the connection
                int MilliSeconds;
                int connection_results = Ctccomm32v2.CtGetRegister(CTconnection, REG_MILLI, &MilliSeconds);
                // Were we successful?
                if (connection_results == SUCCESS)
                {
                    //Yes...
                    statStatus.Text = "Connected";
                    // Enable the timer
                    UpdateTimer.Enabled = true;
                    // Toggle the toolbar buttons.
                    toolDisconnect.Enabled = true;
                    toolConnect.Enabled = false;
                }
            }
        }
       
        private void toolDisconnect_Click(object sender, EventArgs e)
        {
            // Call the function that disconnected to the controller.
            Disconnect();
        }
        private void Disconnect()
        {
            // Are we currently connected?
            if (statStatus.Text == "Connected")
            {
                //Yes, then disconnect
                Ctccomm32v2.CtCloseConnection(CTconnection);
                statStatus.Text = "Disconnected";
                // Disable the timer
                UpdateTimer.Enabled = false;
                // Toggle the toolbar buttons.
                toolDisconnect.Enabled = false;
                toolConnect.Enabled = true;
            }
        }

        private unsafe void UpdateTimer_Tick(object sender, EventArgs e)
        {
            // Read the mSec timer.
            int value;
            if (Ctccomm32v2.CtGetRegister(CTconnection, REG_MILLI, &value) == SUCCESS)
            {
                // Got the reg.
                txtMilliSecTimer.Text = value.ToString();
            }
            else
            {
                // Either the reg is not there, or comm error.
                txtMilliSecTimer.Text = "N/A";
            }
        }

        // Support Functions.
        private void Wait(int duration)
        {
            //Setup the WaitTimer to the duration
            WaitTimer.Interval = duration;
            //Turn on the timer
            WaitTimer.Enabled = true;
            //Wait...
            while (WaitTimer.Enabled == true)
            {
                //Yield to other processes.
                Application.DoEvents();
            }
        }
        private void WaitTimer_Tick(object sender, EventArgs e)
        {
            WaitTimer.Enabled = false;
        }
        private int convertPort(string sPort)
        {
            int port = -1;
            // Does the string contain the text "COMM"?
            if (sPort.ToLower().Contains("comm"))
            {
                // Yes, the remove the text.
                sPort = sPort.Remove(0, 4);
                // Attempt to convert it to an integer.
                if (Int32.TryParse(sPort, out port) == false)
                {
                    // We failed, set it to comm 1;
                    port = 1;
                }
            }
            // Return the results
            return (port);
        }
        private int convertBAUD(string sBaud)
        {
            int baud = 19200;
            // Attempt to convert it to an integer.
            if (Int32.TryParse(sBaud, out baud) == false)
            {
                // We failed, set it to default;
                baud = 19200;
            }
            // Return the results
            return (baud);
        }

        private void MainForm_Load(object sender, EventArgs e)
        {

        }
       
    }
}