using System;
using System.Text;
using System.IO;
using System.Net;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Windows.Forms;
using System.Runtime.InteropServices;


namespace CTCShell
{
    public partial class MainForm : Form
    {
        // All Global Constants
        public const string DEF_PORT = "TCP";
        public const string DEF_BAUD = "19200";
        //public const string DEF_IPADD = "10.0.0.204";
        public const string DEF_IPADD = "172.16.2.241";
        public const string DEF_TIMEOUT = "1000";

        public const int FAILURE = 0;
        public const int SUCCESS = 1;
        
        public const int REG_OS = 13003;
        public const int REG_MILLI = 13002;
        public const int REG_RTCLOCK = 13013;
        public const int REG_SECONDS = 13014;
        public const int REG_MINUTES = 13015;
        public const int REG_HOURS = 13016;
        public const int REG_DAY = 13017;
        public const int REG_MONTH = 13018;
        public const int REG_YEAR = 13019;
        public const int REG_DOW = 13020;
        public const int REG_DOUTS = 1001;
        public const int REG_DOUTS8 = 10201;
        public const int REG_DOUTS16 = 10101;
        public const int REG_DOUTS32 = 10001;
        public const int REG_DINS = 2001;
        public const int REG_DINS8 = 11201;
        public const int REG_DINS16 = 11101;
        public const int REG_DINS32 = 11001;
        
        public int CTconnection;
        public const int rspTimeout = 1000;

        // All Code
        public MainForm()
        {
            InitializeComponent();

            // Set the communication defaults (hardcoded to keep things simple).
            statPort.Text = DEF_PORT;
            statBaud.Text = DEF_BAUD;
            statIPAdd.Text = DEF_IPADD;
            statTimeout.Text = DEF_TIMEOUT;
            statStatus.Text = "Disconnected";
        }

        private void toolConnect_Click(object sender, EventArgs e)
        {
            // Call the function that connects to the controller.
            Connect();
        }
        private unsafe void Connect()
        {
            Int32 timeout;
            // Are we currently disconnected?
            if (statStatus.Text == "Disconnected")
            {
                // Make the proper type connection
                if (statPort.Text == "UDP")
                {
                    // It's a UDP connection...
                    CTconnection = Ctccomm32v2.CtOpenConnection(Ctccomm32v2.CTCUDP, 0, Ctccomm32v2.inet_addr(statIPAdd.Text));
                }
                else if (statPort.Text == "TCP")
                {
                    // It's a TCP connection
                    CTconnection = Ctccomm32v2.CtOpenConnection(Ctccomm32v2.CTCTCP, 0, Ctccomm32v2.inet_addr(statIPAdd.Text));
                }
                else
                {
                    // It's a serial connection
                    CTconnection = Ctccomm32v2.CtOpenConnection(Ctccomm32v2.SERIAL, convertPort(statPort.Text), 1);
                    Ctccomm32v2.CtInitConnection(CTconnection, convertBAUD(statBaud.Text), Ctccomm32v2.NO_PARITY, Ctccomm32v2.EIGHT_DATA_BITS, Ctccomm32v2.ONE_STOP_BITS);
                }
                // Can we convert it to an integer?
                if (Int32.TryParse(statTimeout.Text, out timeout) == true)
                {
                    // Yes, then do it!
                    Ctccomm32v2.CtSetConnectionTimeout(CTconnection, timeout);
                }
                // Test the connection
                int MilliSeconds;
                int connection_results = Ctccomm32v2.CtGetRegister(CTconnection, REG_MILLI, &MilliSeconds);
                // Were we successful?
                if (connection_results == SUCCESS)
                {
                    //Yes...
                    statStatus.Text = "Connected";
                    // Enable the timer
                    UpdateTimer.Enabled = true;
                    // Toggle the toolbar buttons.
                    toolDisconnect.Enabled = true;
                    toolConnect.Enabled = false;
                    // Test code for random read
                    readNRegisters();

                }
            }
        }
       
        private void toolDisconnect_Click(object sender, EventArgs e)
        {
            // Call the function that disconnected to the controller.
            Disconnect();
        }
        private void Disconnect()
        {
            // Are we currently connected?
            if (statStatus.Text == "Connected")
            {
                //Yes, then disconnect
                Ctccomm32v2.CtCloseConnection(CTconnection);
                statStatus.Text = "Disconnected";
                // Disable the timer
                UpdateTimer.Enabled = false;
                // Toggle the toolbar buttons.
                toolDisconnect.Enabled = false;
                toolConnect.Enabled = true;
            }
        }

        private unsafe void UpdateTimer_Tick(object sender, EventArgs e)
        {
            // Read the mSec timer.
            int value;
#if true
            if (Ctccomm32v2.CtGetRegister(CTconnection, REG_MILLI, &value) == SUCCESS)
            {
                // Got the reg.
                txtMilliSecTimer.Text = value.ToString();
            }
            else
            {
                // Either the reg is not there, or comm error.
                txtMilliSecTimer.Text = "N/A";
            }
#endif
        }

        // Support Functions.
        private void Wait(int duration)
        {
            //Setup the WaitTimer to the duration
            WaitTimer.Interval = duration;
            //Turn on the timer
            WaitTimer.Enabled = true;
            //Wait...
            while (WaitTimer.Enabled == true)
            {
                //Yield to other processes.
                Application.DoEvents();
            }
        }
        private void WaitTimer_Tick(object sender, EventArgs e)
        {
            WaitTimer.Enabled = false;
        }
        private int convertPort(string sPort)
        {
            int port = -1;
            // Does the string contain the text "COMM"?
            if (sPort.ToLower().Contains("comm"))
            {
                // Yes, the remove the text.
                sPort = sPort.Remove(0, 4);
                // Attempt to convert it to an integer.
                if (Int32.TryParse(sPort, out port) == false)
                {
                    // We failed, set it to comm 1;
                    port = 1;
                }
            }
            // Return the results
            return (port);
        }
        private int convertBAUD(string sBaud)
        {
            int baud = 19200;
            // Attempt to convert it to an integer.
            if (Int32.TryParse(sBaud, out baud) == false)
            {
                // We failed, set it to default;
                baud = 19200;
            }
            // Return the results
            return (baud);
        }

        private void MainForm_Load(object sender, EventArgs e)
        {

        }

        private void btnReadVReg_Click(object sender, EventArgs e)
        {

            int results;
            // Creat a variant object to pass read request to dll with and get results back in.
            Ctccomm32v2.CT_VARIANT Var =new Ctccomm32v2.CT_VARIANT();
            if (CTconnection == 0)
            {
                // No connection so can't do read
                return;
            }
            // Initialize Variant prior to read
            Var.vRegister = Int32.Parse(txtVRegToRead.Text);      // Variant register to read
            Var.cmd = 0;
            Var.flags = 0;
            Var.slength = 0;
            Var.indexRow = Int32.Parse(txtVRow.Text);
            Var.indexCol = Int32.Parse(txtVCol.Text);
            Var.precision = 6;          // By default a precision of 6
            Var.IntegerIntVar = 0;
            Var.taskHandle = 0;

            if (chkFloat.Checked)
            {
                Var.type = Ctccomm32v2.CT_VARIANT_FLOAT;
            }
            else if (chkDouble.Checked)
            {
                Var.type = Ctccomm32v2.CT_VARIANT_DOUBLE;
            }
            else if (chkString.Checked)
            {
                Var.type = Ctccomm32v2.CT_VARIANT_STRING;
            }
            else
            {
                Var.type = Ctccomm32v2.CT_VARIANT_INTEGER;      // Default
            }
        

            // Since the DLL is marked as unsafe and we use a reference C# will handle the marshalling of data
            results = Ctccomm32v2.CtGetVRegister(CTconnection, ref Var);

            // Read worked properly
            if (results == SUCCESS)
            {
                try
                {
                    if (Var.type == Ctccomm32v2.CT_VARIANT_INTEGER)
                    {
                        txtVRegVal.Text = Convert.ToString(Var.IntegerIntVar);
                    }
                    else if (Var.type == Ctccomm32v2.CT_VARIANT_FLOAT) 
                    {
                        txtVRegVal.Text = Convert.ToString(Var.FloatVar);
                    }
                    else if (Var.type == Ctccomm32v2.CT_VARIANT_DOUBLE) 
                    {
                        txtVRegVal.Text = Convert.ToString(Var.DoubleVar);
                    }
                    else if (Var.type == Ctccomm32v2.CT_VARIANT_STRING) 
                    {
                        if (Var.slength > 0)
                        {
                            unsafe
                            {
                                // Ensure null terminated
                                Var.stringVar[Ctccomm32v2.CT_VARIANT_MAX_STRING] = 0;
                                int len = 0;
                                while (Var.stringVar[len] != 0)
                                    len++;
                                byte[] bytes = new byte[len];
                                for (int i = 0; i < len; i++)
                                    bytes[i] = Var.stringVar[i];
                                // Convert to ASCII
                                txtVRegVal.Text = new ASCIIEncoding().GetString(bytes);
                            }
                        }
                        else
                        {
                            txtVRegVal.Text = "No String?";
                        }
                    }
                    else
                    {
                        txtVRegVal.Text = "ERROR";
                    }
                }
                catch (Exception e1)
                {
                    txtVRegVal.Text = "Bad Data";
                }
            }
            else
            {
                txtVRegVal.Text = "ERROR";
            }
        }

        int readNRegisters()
        {
            int results;
            // Set number of desired registers to read randomly, up to Ctccomm32v2.CT_MAX_RANDOM_REGS
            int regCount = 6;
            // Declare the list to pass what registers to read from
            Ctccomm32v2.CT_REGLIST list =new Ctccomm32v2.CT_REGLIST();
            // Declare the list to return values of each register desired
            Ctccomm32v2.CT_REGLIST regValues = new Ctccomm32v2.CT_REGLIST();

            if (CTconnection == 0)
            {
                // No connection so can't do read
                return -1;
            }
            unsafe
            {
                // Read 6 registers randomly
                list.regList[0] = 1;
                list.regList[1] = 2;
                list.regList[2] = 9;
                list.regList[3] = 8;
                list.regList[4] = 7;
                list.regList[5] = 9;
                // Clear out returned values just for debug purposes
                regValues.regList[0] = 0;
                regValues.regList[1] = 0;
                regValues.regList[2] = 0;
                regValues.regList[3] = 0;
                regValues.regList[4] = 0;
                regValues.regList[5] = 0;
            }
            // Read the registers
            results = Ctccomm32v2.CtGetNRegisters(CTconnection, regCount, ref list, ref regValues);
            return results;
        }

        private void btnWriteVReg_Click(object sender, EventArgs e)
        {

            int results;
            // Creat a variant object to pass read request to dll with and get results back in.
            Ctccomm32v2.CT_VARIANT Var =new Ctccomm32v2.CT_VARIANT();
            if (CTconnection == 0)
            {
                // No connection so can't do read
                return;
            }
            // Initialize Variant prior to read
            Var.vRegister = Int32.Parse(txtVRegToRead.Text);      // Variant register to read
            Var.cmd = 0;
            Var.flags = 0;
            Var.slength = 0;
            Var.indexRow = Int32.Parse(txtVRow.Text);
            Var.indexCol = Int32.Parse(txtVCol.Text);
            Var.precision = 6;          // By default a precision of 6
            Var.IntegerIntVar = 0;
            Var.taskHandle = 0;

            if (chkInteger.Checked)
            {
                Var.type = Ctccomm32v2.CT_VARIANT_INTEGER;
                Var.IntegerIntVar = Convert.ToInt32(txtVRegVal.Text);
            }
            else if (chkFloat.Checked)
            {
                Var.type = Ctccomm32v2.CT_VARIANT_FLOAT;
                Var.FloatVar = Convert.ToSingle(txtVRegVal.Text);
            }
            else if (chkDouble.Checked)
            {
                Var.type = Ctccomm32v2.CT_VARIANT_DOUBLE;
                Var.DoubleVar = Convert.ToDouble(txtVRegVal.Text);
            }
            else if (chkString.Checked)
            {
                Var.type = Ctccomm32v2.CT_VARIANT_STRING;
                Var.slength = txtVRegVal.Text.Length;
                Encoding oEncoder = new System.Text.ASCIIEncoding();
                byte[] tmpArray = oEncoder.GetBytes(txtVRegVal.Text);
//            ' Because of marshalling array must have size of 224 but we copy bytes into it and DLL uses slength
                unsafe
                {
                    for (int i = 0; i != tmpArray.Length; i++)
                    {
                        Var.stringVar[i] = tmpArray[i];
                        Var.stringVar[i + 1] = 0x00;
                    }
                }
            }
        

            // Since the DLL is marked as unsafe and we use a reference C# will handle the marshalling of data
            results = Ctccomm32v2.CtSetVRegister(CTconnection, ref Var);
            if (results == SUCCESS)
            {
               writeStatus.Text = "SUCCESS";
            }
            else
            {
                writeStatus.Text = "ERROR";
            }
        }

        private void txtVRegVal_TextChanged(object sender, EventArgs e)
        {

        }

        private void txtVRegToRead_TextChanged(object sender, EventArgs e)
        {

        }

        private void label2_Click(object sender, EventArgs e)
        {

        }

        private void chkInteger_CheckedChanged(object sender, EventArgs e)
        {
            if (chkInteger.Checked == true) 
            {
                chkFloat.Checked = false;
                chkDouble.Checked = false;
                chkString.Checked = false;
            }
        }

        private void chkFloat__CheckedChanged(object sender, EventArgs e)
        {
            if (chkFloat.Checked == true)
            {
                chkInteger.Checked = false;
                chkDouble.Checked = false;
                chkString.Checked = false;
            }

        }

        private void chkDouble__CheckedChanged(object sender, EventArgs e)
        {
            if (chkDouble.Checked == true)
            {
                chkFloat.Checked = false;
                chkInteger.Checked = false;
                chkString.Checked = false;
            }

        }

        private void chkString_CheckedChanged(object sender, EventArgs e)
        {
            if (chkString.Checked == true)
            {
                chkFloat.Checked = false;
                chkDouble.Checked = false;
                chkInteger.Checked = false;
            }

        }

 
    }
}