using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;


namespace CTCShell
{
    public unsafe class Ctccomm32v2
    {   
    //
    //  MODULE:   ctccom32v2.bas
    //
    //  PURPOSE:
    //    Contains declarations for Ctccom32v2 DLL.
    //

    // Variant access
	public const int CT_VARIANT_ANY = -1;
	public const int CT_VARIANT_INTEGER = 0x0001;
	public const int CT_VARIANT_UINTEGER = 0x0002;
	public const int CT_VARIANT_STRING = 0x0004;
	public const int CT_VARIANT_FLOAT = 0x0008;
	public const int CT_VARIANT_DOUBLE = 0x0010;
    public const int CT_VARIANT_BYTE = 0x80000;   // bit 13

    public const int CT_VARIANT_DOUBLE_NOSWAP_FLAG = 32;
    public const int CT_VARIANT_INDIRECTION_FLAG = 64;

	public const int CT_VARIANT_MAX_STRING = 223;

    [StructLayout(LayoutKind.Explicit, Pack = 1)]
	public unsafe struct CT_VARIANT {
        [FieldOffset(0)]
        public int vRegister;
        [FieldOffset(4)]
		public int type;
		[FieldOffset(8)]
		public int precision;
		[FieldOffset(12)]
		public int flags;
		[FieldOffset(16)]
		public int cmd;
		[FieldOffset(20)]
		public int taskHandle;
		[FieldOffset(24)]
		public int slength;
		[FieldOffset(28)]
		public int indexCol;
		[FieldOffset(32)]
		public int indexRow;
		[FieldOffset(36)]
		public int IntegerIntVar;
		[FieldOffset(40)]
		public float FloatVar;
		[FieldOffset(44)]
		public double DoubleVar;
		[FieldOffset(52)]
		public fixed byte stringVar[CT_VARIANT_MAX_STRING + 1];
    }

	public const int CT_MAX_RANDOM_REGS = 50;

    [StructLayout(LayoutKind.Explicit, Pack = 1)]
	public unsafe struct CT_REGLIST {
		[FieldOffset(0)]
		public fixed int regList[CT_MAX_RANDOM_REGS];
    }

    // constants for binary protocol
    public const int CT_ASCII   = 10;
    public const int CT_REQUEST   = 20;
    public const int CT_REPLY   = 21;
    public const int CT_ACK   = 100;
    public const int CT_NAK   = 101;
    public const int CT_RANGE   = 102;
    public const int CT_VALUE   = 104;

    // function return values
    public const int FAILURE = 0;
    public const int SUCCESS  = 1;
    public const int ERROR_UNKNOWN  = 255;
    public const int ERROR_WIN_TIMER  = 246;
    public const int ERROR_NET_TIMEOUT   = 236;
    public const int ERROR_NET_NOTREADY   = 235;
    public const int ERROR_CT_RANGE   = 216;
    public const int ERROR_BAD_CHECKSUM   = 215;
    public const int ERROR_BAD_LENGTH   = 214;
    public const int ERROR_BAD_RESPONSE   = 206;
    public const int ERROR_NOT_CT_PROTOCOL   = 205;
    public const int ERROR_UNSUPP_REMOTE_OPER   = 196;
    public const int ERROR_UNKNOWN_REM_NODE   = 195;
    public const int ERROR_REMOTE_ACCESS   = 194;
    public const int ERROR_OBJECT_FILE_NOT_FOUND   = 186;
    public const int ERROR_FILE_WRONG_ARCH   = 185;
    public const int ERROR_FILE_NOT_OBJECT   = 184;
    public const int ERROR_BAD_OBJECT_RECORD   = 183;
    public const int ERROR_BAD_OBJECT_LENGTH   = 182;
    public const int ERROR_BAD_POSITION   = 181;
    public const int ERROR_EOF   = 180;
    public const int ERROR_BAD_PORT   = 179;
    public const int ERROR_NET_IN_USE   = 178;
    public const int ERROR_SERIAL_TIMEOUT   = 170;
    public const int ERROR_WARNING   = 169;
    public const int ERROR_ILLEGAL_BAUD_RATE   = 168;
    public const int ERROR_ILLEGAL_PARITY_SETTING   = 167;
    public const int ERROR_ILLEGAL_WORD_LENGTH   = 166;
    public const int ERROR_ILLEGAL_STOP_BITS   = 16;
    public const int ERROR_ERROR   = 160;
    public const int ERROR_PORT_NOT_FOUND   = 159;
    public const int ERROR_PORT_IN_USE   = 158;

    // Constants for port Structures
    public const int NO_PORT   = 0;
    public const int SERIAL   = 1;
    public const int CTCNET   = 2;
    public const int CTCUDP   = 3;
    public const int CTCTCP   = 4;

    // Constants for Parity
    public const byte NO_PARITY = 0;
    public const byte ODD_PARITY = 1;
    public const byte EVEN_PARITY = 2;
    public const byte MARK_PARITY = 3;
    public const byte SPACE_PARITY = 4;

    // Constants for Data Bits
    public const byte FOUR_DATA_BITS = 4;
    public const byte FIVE_DATA_BITS = 5;
    public const byte SIX_DATA_BITS = 6;
    public const byte SEVEN_DATA_BITS = 7;
    public const byte EIGHT_DATA_BITS = 8;

    // Constants for Stop Bits
    public const byte ONE_STOP_BITS = 0;
    public const byte ONE_FIVE_STOP_BITS = 1;
    public const byte TWO_STOP_BITS = 2;

    // network access functions 
    [DllImport("Ctccom32v2.dll")] public static extern int CtOpenConnection 
        (int ConnectID, int CommPort, int Address);
    [DllImport("Ctccom32v2.dll")] public static extern int CtInitConnection
        (int ConnectID, int BaudRate, byte Parity, byte DataBits, byte StopBits);
    [DllImport("Ctccom32v2.dll")] public static extern int CtCloseConnection 
        (int ConnectID);
    [DllImport("Ctccom32v2.dll")] public static extern int CtSetConnectionTimeout
        (int ConnectID, int MilliSeconds);

            // windows functions that may be used
    [DllImport("wsock32.dll")] public static extern int inet_addr
        (string ipAddress);      

    // register functions
    [DllImport("Ctccom32v2.dll")] public static extern int CtGetRegister 
        (int ConnectID, int RegAddress, int *RegValue);
    [DllImport("Ctccom32v2.dll")] public static extern int CtPutRegister
        (int ConnectID, int RegAddress, int RegValue);

    [DllImport("Ctccom32v2.dll")] public unsafe static extern int CtGetVProperties
        (int ConnectID, int RegAddress, int *pSizeY_Row, int *pSizeX_Column, int *pPrecision);

    [DllImport("Ctccom32v2.dll")] public unsafe static extern int CtGetVRegister
        (int ConnectID, ref CT_VARIANT Storage);

    [DllImport("Ctccom32v2.dll")] public unsafe static extern int CtSetVRegister
       (int ConnectID, ref CT_VARIANT Storage);

    [DllImport("Ctccom32v2.dll")] public unsafe static extern int CtGetNRegisters
        (int ConnectID, int regCount, ref CT_REGLIST regList, ref CT_REGLIST values);

 }
}
